@@ -16,8 +16,14 @@
 from __future__ import annotations
 
 import sys
+import warnings
 
-from . import patches
+if sys.flags.dev_mode and not sys.warnoptions:
+    warnings.simplefilter("error", DeprecationWarning)
+warnings.filterwarnings("ignore", module="defusedxml")
+
+
+from . import patches  # pylint: disable=wrong-import-position  # noqa: E402
 
 patches.apply()
 
