@@ -19,11 +19,7 @@ from redis.asyncio import Redis
 from tornado.web import HTTPError
 
 from .. import EVENT_REDIS
-from ..utils.request_handler import (
-    APIRequestHandler,
-    HTMLRequestHandler,
-    NotFoundHandler,
-)
+from ..utils.request_handler import APIRequestHandler, HTMLRequestHandler
 from ..utils.utils import ModuleInfo
 
 
@@ -32,11 +28,9 @@ def get_module_info() -> ModuleInfo:
     return ModuleInfo(
         handlers=(
             (r"/LOLWUT", LOLWUT),
-            (r"/LOLWUT/([0-9/]+)/", NotFoundHandler),
-            (r"/LOLWUT/([0-9/]+)", LOLWUT),
+            (r"/LOLWUT/((?:\d+/)*\d+)", LOLWUT),
             (r"/API/LOLWUT", LOLWUTAPI),
-            (r"/API/LOLWUT/([0-9/]+)/", NotFoundHandler),
-            (r"/API/LOLWUT/([0-9/]+)", LOLWUTAPI),
+            (r"/API/LOLWUT/((?:\d+/)*\d+)", LOLWUTAPI),
         ),
         name="LOLWUT",
         description="LOLWUT; präsentiert von Redis",
@@ -74,9 +68,12 @@ class LOLWUT(HTMLRequestHandler):
         """Handle GET requests to the LOLWUT page."""
         if not EVENT_REDIS.is_set():
             raise HTTPError(503)
+
         art = await generate_art(self.redis, args, head)
+
         if head:
             return
+
         await self.render(
             "ansi2html.html",
             ansi=art,
@@ -97,9 +94,10 @@ class LOLWUTAPI(APIRequestHandler):
         """Handle GET requests to the LOLWUT API."""
         if not EVENT_REDIS.is_set():
             raise HTTPError(503)
+
         art = await generate_art(self.redis, args, head)
+
         if self.content_type == "text/plain":
-            self.set_header("Content-Type", "text/plain; charset=UTF-8")
             return await self.finish(art)
 
         await self.finish({"LOLWUT": art})
