@@ -31,7 +31,7 @@ from tornado.web import HTTPError, RedirectHandler
 
 from .. import EVENT_REDIS
 from ..main import IGNORED_MODULES
-from ..utils.request_handler import APIRequestHandler
+from ..utils.request_handler import APIRequestHandler, HTMLRequestHandler
 from ..utils.utils import ModuleInfo, hash_ip
 from . import (
     WRONG_QUOTES_CACHE,
@@ -47,7 +47,7 @@ from .create import get_module_info as get_create_mi
 from .quote_generator import get_module_info as get_generator_mi
 from .quote_of_the_day import QuoteOfTheDayBaseHandler
 from .quote_of_the_day import get_module_info as get_qod_mi
-from .quotes_image import QuoteAsImage
+from .quotes_image import QuoteAsImage, create_image
 from .share_page import ShareQuote
 
 logger = logging.getLogger(__name__)
@@ -85,6 +85,10 @@ def get_module_info() -> ModuleInfo:
                 r"/zitate/([0-9]{1,10})-([0-9]{1,10})\.([a-zA-Z]{3,4})",
                 QuoteAsImage,
             ),
+            (
+                r"/zitate/([0-9]{1,10})()\.([a-zA-Z]{3,4})",
+                QuoteAsImage,
+            ),
             (  # redirect to the new URL (changed because of robots.txt)
                 r"/zitate/([0-9]{1,10})-([0-9]{1,10})/share",
                 RedirectHandler,
@@ -96,6 +100,10 @@ def get_module_info() -> ModuleInfo:
                 r"/api/zitate/([0-9]{1,10})-([0-9]{1,10})(?:/full|)",
                 QuoteAPIHandler,
             ),
+            (
+                r"/api/zitate/([0-9]{1,10})(?:/full|)",
+                QuoteAPIHandler,
+            ),
         ),
         name="Falsch zugeordnete Zitate",
         short_name="Falsche Zitate",
@@ -150,7 +158,7 @@ class QuoteBaseHandler(QuoteReadyCheckHandler):
 
     TASK_REFERENCES: set[Task[Any]] = set()
 
-    def __init__(self, *args, **kwargs):  # type: ignore
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
         """Initialize the base request handler."""
         super().__init__(*args, **kwargs)
         self.awaitables: list[
@@ -307,17 +315,23 @@ class QuoteById(QuoteBaseHandler):
     """The page with a specified quote that then gets rendered."""
 
     RATELIMIT_GET_LIMIT = 10
-    RATELIMIT_GET_COUNT_PER_PERIOD = 15  # 15 requests per 10s
+    RATELIMIT_GET_COUNT_PER_PERIOD = 15
     RATELIMIT_GET_PERIOD = 10
 
     RATELIMIT_POST_LIMIT = 5
-    RATELIMIT_POST_COUNT_PER_PERIOD = 10  # 10 requests per 30s
+    RATELIMIT_POST_COUNT_PER_PERIOD = 10
     RATELIMIT_POST_PERIOD = 30
 
+    POSSIBLE_CONTENT_TYPES = (
+        *HTMLRequestHandler.POSSIBLE_CONTENT_TYPES,
+        *QuoteAsImage.POSSIBLE_CONTENT_TYPES,
+    )
+    LONG_PATH = "/zitate/%d-%d"
+
     async def get(
         self, quote_id: str, author_id: None | str = None, *, head: bool = False
     ) -> None:
-        """Handle the GET request to this page and render the quote."""
+        """Handle GET requests to this page and render the quote."""
         int_quote_id = int(quote_id)
         if author_id is None:
             wqs = get_wrong_quotes(lambda wq: wq.id == int_quote_id)
@@ -325,7 +339,18 @@ class QuoteById(QuoteBaseHandler):
                 raise HTTPError(404, f"No wrong quote with id {quote_id}")
             return self.redirect(
                 self.fix_url(
-                    f"/zitate/{wqs[0].quote.id}-{wqs[0].author.id}",
+                    self.LONG_PATH % (wqs[0].quote.id, wqs[0].author.id)
+                )
+            )
+        if self.content_type and self.content_type.startswith("image/"):
+            _wq = await get_wrong_quote(int_quote_id, int(author_id))
+            return await self.finish(
+                create_image(
+                    _wq.quote.quote,
+                    _wq.author.name,
+                    _wq.rating,
+                    f"{self.request.host_name}/z/{_wq.get_id_as_str(True)}",
+                    self.content_type.removeprefix("image/"),
                 )
             )
         if head:
@@ -334,7 +359,7 @@ class QuoteById(QuoteBaseHandler):
 
     async def post(self, quote_id_str: str, author_id_str: str) -> None:
         """
-        Handle the POST request to this page and render the quote.
+        Handle POST requests to this page and render the quote.
 
         This is used to vote the quote, without changing the URL.
         """
@@ -471,10 +496,16 @@ class QuoteAPIHandler(APIRequestHandler, QuoteById):
     """API request handler for the quotes page."""
 
     RATELIMIT_GET_PERIOD = 10
-    RATELIMIT_GET_COUNT_PER_PERIOD = 20  # 20 requests per 10s
+    RATELIMIT_GET_COUNT_PER_PERIOD = 20
 
     ALLOWED_METHODS = ("GET", "POST")
 
+    POSSIBLE_CONTENT_TYPES = (
+        *APIRequestHandler.POSSIBLE_CONTENT_TYPES,
+        *QuoteAsImage.POSSIBLE_CONTENT_TYPES,
+    )
+    LONG_PATH = "/api/zitate/%d-%d"
+
     async def render_wrong_quote(
         self, wrong_quote: WrongQuote, vote: int
     ) -> None:
