@@ -32,10 +32,25 @@ from urllib.parse import SplitResult, parse_qsl, urlencode, urlsplit, urlunsplit
 import elasticapm  # type: ignore
 from blake3 import blake3  # type: ignore
 from elasticsearch import AsyncElasticsearch
+from Levenshtein import distance  # type: ignore
 from tornado.web import HTTPError, RequestHandler
 
 from .. import STATIC_DIR
 
+# All the text content types used by the website
+TEXT_CONTENT_TYPES = {
+    "application/javascript",
+    "application/json",
+    "application/rss+xml",
+    "application/x-ndjson",
+    "application/xml",
+    "application/yaml",
+    "image/svg+xml",
+    "text/html",
+    "text/markdown",
+    "text/plain",
+}
+
 T = TypeVar("T")
 
 # pylint: disable=consider-alternative-union-syntax
@@ -187,13 +202,14 @@ class Timer:
         return self._execution_time
 
 
-class Permissions(IntFlag):
+class Permission(IntFlag):
     """Permissions for accessing the website."""
 
     RATELIMITS = 1
     TRACEBACK = 2
     BACKDOOR = 4
     UPDATE = 8
+    REPORTING = 16
 
 
 @cache
@@ -281,6 +297,11 @@ def bool_to_str(val: bool) -> str:
     return "sure" if val else "nope"
 
 
+def country_code_to_flag(code: str) -> str:
+    """Convert a two-letter ISO country code to a flag emoji."""
+    return "".join(chr(ord(char) + 23 * 29 * 191) for char in code.upper())
+
+
 def emojify(string: str) -> str:
     """Emojify a given string."""
     string = re.sub(
@@ -301,11 +322,6 @@ def emojify(string: str) -> str:
     )
 
 
-def country_code_to_flag(code: str) -> str:
-    """Convert a two-letter ISO country code to a flag emoji."""
-    return "".join(chr(ord(char) + 23 * 29 * 191) for char in code.upper())
-
-
 async def geoip(
     ip: str,  # pylint: disable=invalid-name
     database: str = "GeoLite2-City.mmdb",
@@ -422,6 +438,11 @@ def name_to_id(val: str) -> str:
     ).strip("-")
 
 
+def normalized_levenshtein(string1: str, string2: str) -> float:
+    """Calculate the normalized Levenshtein distance between two strings."""
+    return float(distance(string1, string2)) / max(len(string1), len(string2))
+
+
 def replace_umlauts(string: str) -> str:
     """Replace Ä, Ö, Ü, ẞ, ä, ö, ü, ß in string."""
     if string.isupper():
